@@ -1,12 +1,13 @@
 import csv
 from datetime import datetime
 from pathlib import Path
-from typing import List, Optional, NamedTuple
+from typing import List, Optional, NamedTuple, Tuple
 from urllib.parse import ParseResult as URLParseResult
 
 import click
 
-from sparql_query_tools.RunQuery import run_queries
+from sparql_query_tools.ParseSparqlJsonResult import SparqlJsonResultStats
+from sparql_query_tools.RunQuery import run_queries, QueryResult
 from sparql_query_tools.clickextension.IntList import IntList
 from sparql_query_tools.clickextension.QueryFilesParam import QueriesFileParam
 from sparql_query_tools.clickextension.URLParam import URLParam
@@ -46,11 +47,18 @@ class OutputCSVRows(NamedTuple):
               help="Don't write the results to disk.")
 @click.option('--save-only-error', '-sor', is_flag=True, required=False, type=bool, default=False,
               help="Save the sparql result file only in case of an error.")
+@click.option('--dont-parse', '-dp', is_flag=True, required=False, type=bool, default=False,
+              help="Don't parse the result. HTTP Content-Length is still reported.")
+@click.option('--time-out', '-t', required=False, type=int, default=600000,
+              help="Timeout in milliseconds. Default is 10 minutes.")
 @click.option('--output', '-o', required=False, type=str, default=None,
               help='Custom location for output csv file.'
                    'If set the result files are written in a directory next to the csv file with the same name.')
+@click.option('--verbose', '-v', is_flag=True, required=False, type=bool, default=False, help='verbose logging')
 def cli(url: URLParseResult, queries: Path, include: Optional[List[int]], exclude: Optional[List[int]],
-        storename: Optional[str], datasetname: Optional[str], save=False, save_only_error=False, output=None):
+        storename: Optional[str], datasetname: Optional[str], save: bool, save_only_error: bool, dont_parse: bool,
+        time_out: int,
+        output: Optional[str], verbose: bool):
     if output is not None:
         if Path(output).suffix != ".csv":
             click.echo("Output file extension must be .csv")
@@ -75,16 +83,34 @@ def cli(url: URLParseResult, queries: Path, include: Optional[List[int]], exclud
     query_ids, queries = parse_queries_file(queries, include, exclude)
 
     csv_path: Path = output_dir.joinpath(output_name + ".csv")
-    print(str(csv_path))
+
+    results: List[Tuple[int, str, QueryResult, Optional[SparqlJsonResultStats]]] = list()
+    if save:
+        if not save_only_error:
+            results = [*run_queries(query_ids, queries, url, result_files_dir, parse=not dont_parse, timeout_ms=time_out, verbose=verbose)]
+        else:
+            for result in run_queries(query_ids, queries, url, result_files_dir, parse=not dont_parse, timeout_ms=time_out, verbose=verbose):
+                results.append(result)
+                query_id, query, download_result, parse_result = result
+                if download_result.status == 200 or (
+                        download_result.status == 200 and not dont_parse and parse_result.success):
+                    if download_result.path.exists():
+                        download_result.path.unlink()
+    else:
+        if dont_parse:
+            results = [*run_queries(query_ids, queries, url, None, parse=False, timeout_ms=time_out, verbose=verbose)]
+        else:
+            for result in run_queries(query_ids, queries, url, result_files_dir, parse=True, timeout_ms=time_out, verbose=verbose):
+                results.append(result)
+                query_id, query, download_result, parse_result = result
+                if download_result.path.exists():
+                    download_result.path.unlink()
+
     with open(csv_path, 'w', newline='') as csvfile:
         writer = csv.DictWriter(csvfile, fieldnames=OutputCSVRows._fields)
         writer.writeheader()
-        for query_id, query, download_result, parse_result in run_queries(query_ids, queries, url, result_files_dir):
-            http_succeeded = download_result.status == 200
-            parsing_succeeded: bool = parse_result.success
-            if not save or (save_only_error and http_succeeded and parsing_succeeded):
-                if download_result.path.exists():
-                    download_result.path.unlink()
+        for query_id, query, download_result, parse_result in results:
+            http_succeeded = download_result.status == 200 and download_result.error_message is None
 
             row = OutputCSVRows(
                 format="HTTP",
@@ -98,16 +124,16 @@ def cli(url: URLParseResult, queries: Path, include: Optional[List[int]], exclud
                 errorMessage=download_result.error_message,
                 time=download_result.duration_s,
                 contentLength=download_result.file_size_bytes,
-                parsingSucceeded=parsing_succeeded if http_succeeded else None,
-                numberOfVariables=parse_result.no_of_variables if http_succeeded else None,
-                numberOfSolutions=parse_result.no_of_solutions if http_succeeded else None,
-                numberOfBindings=parse_result.no_of_var_bindings if http_succeeded else None,
-                resultParsingTime=parse_result.parse_duration if http_succeeded else None,
-                parsingErrorMessage=parse_result.error_message if http_succeeded else None,
+                parsingSucceeded=parse_result.success if http_succeeded and parse_result else None,
+                numberOfVariables=parse_result.no_of_variables if http_succeeded and parse_result else None,
+                numberOfSolutions=parse_result.no_of_solutions if http_succeeded and parse_result else None,
+                numberOfBindings=parse_result.no_of_var_bindings if http_succeeded and parse_result else None,
+                resultParsingTime=parse_result.parse_duration if http_succeeded and parse_result else None,
+                parsingErrorMessage=parse_result.error_message if http_succeeded and parse_result else None,
             )
             writer.writerow(row._asdict())
-        if not any(result_files_dir.iterdir()):
-            result_files_dir.rmdir()
+    if not any(result_files_dir.iterdir()):
+        result_files_dir.rmdir()
 
 
 if __name__ == '__main__':
